#!/usr/bin/python3
# -*- coding: UTF-8 -*-

## masterGeo_init.py

#############################################################################
# Copyright (C) Labomedia December 2013
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franproplin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
#############################################################################

from time import time
import json
from math import sin, cos, radians, log2
from random import random
from collections import OrderedDict
from pprint import pprint
from bge import logic as gl
from bge import render

# Global variable
RAYON = 319.6
DELTA = 0.01
OPTIMISED = True
STEP = 10
FILE = "//my_data.json"

# Fixed global variable
gl.T = time()

def master_init_main():
    scene = gl.getCurrentScene()

    cam = scene.active_camera
    set_camera_Loc_Rot(cam)

    read_json()

    objList = scene.objects
    empty = objList["Empty"]
    display_all_triangle(empty)
    gl.mouse_position = (0, 0)
    
def set_camera_Loc_Rot(cam):
    cam.applyRotation([-0.21, -0.0, 0.12], True)
    # Center the mouse
    render.setMousePosition(int(render.getWindowWidth() / 2),
                                int(render.getWindowHeight() / 2))

def read_json():
    json_file = gl.expandPath(FILE)
    json_data = open(json_file)
    data = json.load(json_data)
    json_data.close()

    gl.all_data = OrderedDict()
    i = 0
    for point in data['locations']:
        gl.all_data[i] = (int(point['latitudeE7']/10000),
                        int(point['longitudeE7']/10000))
        i += 1
    t = time() - gl.T
    gl.T = time()
    print("Le fichier {0} a été lu en {1} secondes".format(FILE, t))
    print("Nombre de points dans ce fichier = {0}".format(i))

def display_all_triangle(empty):
    n = len(gl.all_data)
    displayed = []

    for i in range(0, n, STEP):
        x,y,z = angle_to_ortho(i)
        add = True
        if OPTIMISED:
            for val in displayed:
                d = ((x-val[0])**2 + (y-val[1])**2 + (z-val[2])**2)**0.5
                if d < DELTA:
                    add = False
                    val[3] += 1
        else:
            add = True

        if add:
            displayed.insert(0, [x, y, z, 1])
            print("Point numéro {0} ajouté".format(len(displayed)))

    for p in displayed:
        x,y,z,h = p[0], p[1], p[2], p[3]
        add_triangle(x, y, z,h, empty)
    t = time() - gl.T
    print("{0} points affichés en {1} secondes".format(len(displayed), t))

def add_triangle(x, y, z, h, empty):
    scene = gl.getCurrentScene()

    empty.worldPosition = (x, y, z)
    empty.localScale = 0.3, 0.3, 1 + log2(h)
    ##rotation = [0.0, 0.0, 0.1]
    ##empty.applyRotation = (rotation, False)
    life = 0
    # Add  during life frames: 0 = infinite
    scene.addObject("triangle", empty, life)

def angle_to_ortho(i):
    a = gl.all_data[i][0]/1000
    b = gl.all_data[i][1]/1000

    latit  = radians(a)
    longit = radians(b)

    r = RAYON
    x = r * cos(latit) * cos(longit)
    z = r * sin(latit)
    y = (r**2 - x**2 - z**2)**0.5

    return x,y,z
